@@ -1,10 +1,10 @@
 /*
- *  $Id: libnet_build_icmp.c,v 1.9 2003/11/03 03:01:22 mike Exp $
+ *  $Id: libnet_build_icmp.c,v 1.15 2004/03/25 18:50:48 mike Exp $
  *
  *  libnet
  *  libnet_build_icmp.c - ICMP packet assemblers
  *
- *  Copyright (c) 1998 - 2003 Mike D. Schiffman <mike@infonexus.com>
+ *  Copyright (c) 1998 - 2004 Mike D. Schiffman <mike@infonexus.com>
  *  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -39,10 +39,42 @@
 #include "../include/win32/libnet.h"
 #endif
 
+/* some common cruft for completing ICMP error packets */
+#define LIBNET_BUILD_ICMP_ERR_FINISH(type)                                   \
+do                                                                           \
+{                                                                            \
+    n = libnet_pblock_append(l, p, (u_int8_t *)&icmp_hdr, type);             \
+    if (n == -1)                                                             \
+    {                                                                        \
+        goto bad;                                                            \
+    }                                                                        \
+                                                                             \
+    if ((payload && !payload_s) || (!payload && payload_s))                  \
+    {                                                                        \
+        snprintf(l->err_buf, LIBNET_ERRBUF_SIZE,                             \
+                "%s(): payload inconsistency\n", __func__);                  \
+        goto bad;                                                            \
+    }                                                                        \
+                                                                             \
+    if (payload && payload_s)                                                \
+    {                                                                        \
+        n = libnet_pblock_append(l, p, payload, payload_s);                  \
+        if (n == -1)                                                         \
+        {                                                                    \
+            goto bad;                                                        \
+        }                                                                    \
+    }                                                                        \
+                                                                             \
+    if (sum == 0)                                                            \
+    {                                                                        \
+        libnet_pblock_setflags(p, LIBNET_PBLOCK_DO_CHECKSUM);                \
+    }                                                                        \
+} while (0)
+
 libnet_ptag_t
 libnet_build_icmpv4_echo(u_int8_t type, u_int8_t code, u_int16_t sum,
-            u_int16_t id, u_int16_t seq, u_int8_t *payload,
-            u_int32_t payload_s, libnet_t *l, libnet_ptag_t ptag) 
+u_int16_t id, u_int16_t seq, u_int8_t *payload, u_int32_t payload_s,
+libnet_t *l, libnet_ptag_t ptag) 
 {
     u_int32_t n, h;
     libnet_pblock_t *p;
@@ -66,16 +98,12 @@ libnet_build_icmpv4_echo(u_int8_t type, u_int8_t code, u_int16_t sum,
         return (-1);
     }
 
-	memset(&icmp_hdr, 0, sizeof(icmp_hdr));
-	icmp_hdr.icmp_type = type;          /* packet type */
+    memset(&icmp_hdr, 0, sizeof(icmp_hdr));
+    icmp_hdr.icmp_type = type;          /* packet type */
     icmp_hdr.icmp_code = code;          /* packet code */
-    icmp_hdr.icmp_sum  = sum;           /* checksum */
-    /*
-     *  XXX - Does this need to be Big Endian or what?  I was testing on
-     *  04/23/2002 and stuff was breaking with these htons()'d.
-     */
-    icmp_hdr.icmp_id   = id;            /* packet id */
-    icmp_hdr.icmp_seq  = seq;           /* packet seq */
+    icmp_hdr.icmp_sum  = (sum ? htons(sum) : 0);  /* checksum */
+    icmp_hdr.icmp_id   = htons(id);            /* packet id */
+    icmp_hdr.icmp_seq  = htons(seq);           /* packet seq */
 
     n = libnet_pblock_append(l, p, (u_int8_t *)&icmp_hdr, LIBNET_ICMPV4_ECHO_H);
     if (n == -1)
@@ -86,7 +114,7 @@ libnet_build_icmpv4_echo(u_int8_t type, u_int8_t code, u_int16_t sum,
     if ((payload && !payload_s) || (!payload && payload_s))
     {
          snprintf(l->err_buf, LIBNET_ERRBUF_SIZE,
-			     "%s(): payload inconsistency\n", __func__);
+                 "%s(): payload inconsistency\n", __func__);
         goto bad;
     }
  
@@ -115,11 +143,10 @@ bad:
     return (-1);
 }
 
-
 libnet_ptag_t
 libnet_build_icmpv4_mask(u_int8_t type, u_int8_t code, u_int16_t sum,
-            u_int16_t id, u_int16_t seq, u_int32_t mask, u_int8_t *payload,
-            u_int32_t payload_s, libnet_t *l, libnet_ptag_t ptag)
+u_int16_t id, u_int16_t seq, u_int32_t mask, u_int8_t *payload,
+u_int32_t payload_s, libnet_t *l, libnet_ptag_t ptag)
 {
     u_int32_t n, h;
     libnet_pblock_t *p;
@@ -143,10 +170,10 @@ libnet_build_icmpv4_mask(u_int8_t type, u_int8_t code, u_int16_t sum,
         return (-1);
     }
 
-	memset(&icmp_hdr, 0, sizeof(icmp_hdr));
-	icmp_hdr.icmp_type = type;          /* packet type */
+    memset(&icmp_hdr, 0, sizeof(icmp_hdr));
+    icmp_hdr.icmp_type = type;          /* packet type */
     icmp_hdr.icmp_code = code;          /* packet code */
-    icmp_hdr.icmp_sum  = sum;           /* checksum */
+    icmp_hdr.icmp_sum  = (sum ? htons(sum) : 0);  /* checksum */
     icmp_hdr.icmp_id   = htons(id);     /* packet id */
     icmp_hdr.icmp_seq  = htons(seq);    /* packet seq */
     icmp_hdr.icmp_mask = htonl(mask);   /* address mask */
@@ -189,12 +216,10 @@ bad:
     return (-1);
 }
 
-
 libnet_ptag_t
 libnet_build_icmpv4_timestamp(u_int8_t type, u_int8_t code, u_int16_t sum,
-            u_int16_t id, u_int16_t seq, n_time otime, n_time rtime,
-            n_time ttime, u_int8_t *payload, u_int32_t payload_s, libnet_t *l,
-            libnet_ptag_t ptag)
+u_int16_t id, u_int16_t seq, n_time otime, n_time rtime, n_time ttime,
+u_int8_t *payload, u_int32_t payload_s, libnet_t *l, libnet_ptag_t ptag)
 {
     u_int32_t n, h;
     libnet_pblock_t *p;
@@ -218,10 +243,10 @@ libnet_build_icmpv4_timestamp(u_int8_t type, u_int8_t code, u_int16_t sum,
         return (-1);
     }
 
-	memset(&icmp_hdr, 0, sizeof(icmp_hdr));
-	icmp_hdr.icmp_type  = type;             /* packet type */
+    memset(&icmp_hdr, 0, sizeof(icmp_hdr));
+    icmp_hdr.icmp_type  = type;             /* packet type */
     icmp_hdr.icmp_code  = code;             /* packet code */
-    icmp_hdr.icmp_sum   = sum;              /* checksum */
+    icmp_hdr.icmp_sum   = (sum ? htons(sum) : 0); /* checksum */
     icmp_hdr.icmp_id    = htons(id);        /* packet id */
     icmp_hdr.icmp_seq   = htons(seq);       /* packet seq */
     icmp_hdr.icmp_otime = htonl(otime);     /* original timestamp */
@@ -266,67 +291,26 @@ bad:
     return (-1);
 }
 
-
 libnet_ptag_t
 libnet_build_icmpv4_unreach(u_int8_t type, u_int8_t code, u_int16_t sum,
-            u_int16_t orig_len, u_int8_t orig_tos, u_int16_t orig_id,
-            u_int16_t orig_frag, u_int8_t orig_ttl, u_int8_t orig_prot,
-            u_int16_t orig_check, u_int32_t orig_src, u_int32_t orig_dst,
-            u_int8_t *payload, u_int32_t payload_s, libnet_t *l,
-            libnet_ptag_t ptag)
+u_int8_t *payload, u_int32_t payload_s, libnet_t *l, libnet_ptag_t ptag)
 {
     u_int32_t n, h;
-    libnet_ptag_t ipv4;
-    libnet_pblock_t *p, *q;
+    libnet_pblock_t *p;
     struct libnet_icmpv4_hdr icmp_hdr;
 
     if (l == NULL)
     { 
         return (-1);
     } 
+    n = LIBNET_ICMPV4_UNREACH_H + payload_s;        /* size of memory block */
 
-    p = NULL;
-    /* payload refers to the payload in the original IP header */
-    if ((payload && !payload_s) || (!payload && payload_s))
-    {
-         snprintf(l->err_buf, LIBNET_ERRBUF_SIZE,
-			     "%s(): payload inconsistency\n", __func__);
-        goto bad;
-    }
-
-    /*
-     *  Build the original IPv4 header in its own pblock.  We stick the
-     *  payload here.
+    /* 
+     * FREDRAYNAL: as ICMP checksum includes what is embedded in 
+     * the payload, and what is after the ICMP header, we need to include
+     * those 2 sizes.
      */
-    ipv4 = libnet_build_ipv4(orig_len, orig_tos, orig_id, orig_frag, orig_ttl,
-        orig_prot, orig_check, orig_src, orig_dst, payload, payload_s, l, 0);
-
-    if (ipv4 == -1)
-    {
-        /* error set elsewhere */
-        return (-1);
-    }
-
-    if (orig_check == 0)
-    {
-       int c = 0;
-       l->aligner = 8 - (l->link_offset % 8);
-       q = libnet_pblock_find(l, ipv4);
-       c = libnet_do_checksum(l, q->buf, libnet_pblock_p2p(q->type),
-                              q->h_len);
-       if (c == -1)
-       {
-           /* err msg set in libnet_do_checksum() */
-           return (-1);
-       }
-    }
-
-    /*
-     * Now we can build the ICMP part of the packet
-     */
-    n = LIBNET_ICMPV4_UNREACH_H;        /* size of memory block */
-    /* hl for checksum */
-    h = LIBNET_ICMPV4_UNREACH_H + LIBNET_IPV4_H + payload_s;
+    h = LIBNET_ICMPV4_UNREACH_H + payload_s + l->total_size; 
 
     /*
      *  Find the existing protocol block if a ptag is specified, or create
@@ -338,30 +322,15 @@ libnet_build_icmpv4_unreach(u_int8_t type, u_int8_t code, u_int16_t sum,
         return (-1);
     }
 
-	memset(&icmp_hdr, 0, sizeof(icmp_hdr));
-	icmp_hdr.icmp_type = type;          /* packet type */
+    memset(&icmp_hdr, 0, sizeof(icmp_hdr));
+    icmp_hdr.icmp_type = type;          /* packet type */
     icmp_hdr.icmp_code = code;          /* packet code */
-    icmp_hdr.icmp_sum  = sum;           /* checksum */
+    icmp_hdr.icmp_sum  = (sum ? htons(sum) : 0);  /* checksum */
     icmp_hdr.icmp_id   = 0;             /* must be 0 */
     icmp_hdr.icmp_seq  = 0;             /* must be 0 */
 
-    n = libnet_pblock_append(l, p, (u_int8_t *)&icmp_hdr,
-            LIBNET_ICMPV4_UNREACH_H);
-    if (n == -1)
-    {
-        /* error set elsewhere */
-        goto bad;
-    }
+    LIBNET_BUILD_ICMP_ERR_FINISH(LIBNET_ICMPV4_UNREACH_H);
 
-    if (sum == 0)
-    {
-        /*
-         *  If checksum is zero, by default libnet will compute a checksum
-         *  for the user.  The programmer can override this by calling
-         *  libnet_toggle_checksum(l, ptag, 1);
-         */
-        libnet_pblock_setflags(p, LIBNET_PBLOCK_DO_CHECKSUM);
-    }
     return (ptag ? ptag : libnet_pblock_update(l, p, h,
             LIBNET_PBLOCK_ICMPV4_UNREACH_H));
 bad:
@@ -369,18 +338,12 @@ bad:
     return (-1);
 }
 
-
 libnet_ptag_t
 libnet_build_icmpv4_timeexceed(u_int8_t type, u_int8_t code, u_int16_t sum,
-            u_int16_t orig_len, u_int8_t orig_tos, u_int16_t orig_id,
-            u_int16_t orig_frag, u_int8_t orig_ttl, u_int8_t orig_prot,
-            u_int16_t orig_check, u_int32_t orig_src, u_int32_t orig_dst,
-            u_int8_t *payload, u_int32_t payload_s, libnet_t *l,
-            libnet_ptag_t ptag)
+u_int8_t *payload, u_int32_t payload_s, libnet_t *l, libnet_ptag_t ptag)
 {
     u_int32_t n, h;
-    libnet_ptag_t ipv4;
-    libnet_pblock_t *p, *q;
+    libnet_pblock_t *p;
     struct libnet_icmpv4_hdr icmp_hdr;
 
     if (l == NULL)
@@ -388,46 +351,14 @@ libnet_build_icmpv4_timeexceed(u_int8_t type, u_int8_t code, u_int16_t sum,
         return (-1);
     } 
 
-    p = NULL;
-    /* payload refers to the payload in the original IP header */
-    if ((payload && !payload_s) || (!payload && payload_s))
-    {
-         snprintf(l->err_buf, LIBNET_ERRBUF_SIZE,
-			     "%s(): payload inconsistency\n", __func__);
-        goto bad;
-    }
-
-    /*
-     *  Build the original IPv4 header in its own pblock.  We stick the
-     *  payload here.
-     */
-    ipv4 = libnet_build_ipv4(orig_len, orig_tos, orig_id, orig_frag, orig_ttl,
-        orig_prot, orig_check, orig_src, orig_dst, payload, payload_s, l, 0);
-
-    if (ipv4 == -1)
-    {
-        /* error set elsewhere */
-	return (-1);
-    }
-
-    if (orig_check == 0)
-    {
-       int c = 0;
-       l->aligner = 8 - (l->link_offset % 8);
-       q = libnet_pblock_find(l, ipv4);
-       c = libnet_do_checksum(l, q->buf, libnet_pblock_p2p(q->type),
-                              q->h_len);
-       if (c == -1)
-       {
-           /* err msg set in libnet_do_checksum() */
-           return (-1);
-       }
-    }
-
     /* size of memory block */
     n = LIBNET_ICMPV4_TIMXCEED_H;
-    /* hl for checksum */
-    h = LIBNET_ICMPV4_TIMXCEED_H + LIBNET_IPV4_H + payload_s;
+    /* 
+     * FREDRAYNAL: as ICMP checksum includes what is embedded in 
+     * the payload, and what is after the ICMP header, we need to include
+     * those 2 sizes.
+     */
+    h = LIBNET_ICMPV4_TIMXCEED_H + payload_s + l->total_size; 
 
     /*
      *  Find the existing protocol block if a ptag is specified, or create
@@ -439,29 +370,15 @@ libnet_build_icmpv4_timeexceed(u_int8_t type, u_int8_t code, u_int16_t sum,
         return (-1);
     }
 
-	memset(&icmp_hdr, 0, sizeof(icmp_hdr));
-	icmp_hdr.icmp_type = type;          /* packet type */
+    memset(&icmp_hdr, 0, sizeof(icmp_hdr));
+    icmp_hdr.icmp_type = type;          /* packet type */
     icmp_hdr.icmp_code = code;          /* packet code */
-    icmp_hdr.icmp_sum  = sum;           /* checksum */
+    icmp_hdr.icmp_sum  = (sum ? htons(sum) : 0);  /* checksum */
     icmp_hdr.icmp_id   = 0;             /* must be 0 */
     icmp_hdr.icmp_seq  = 0;             /* must be 0 */
 
-    n = libnet_pblock_append(l, p, (u_int8_t *)&icmp_hdr,
-            LIBNET_ICMPV4_TIMXCEED_H);
-    if (n == -1)
-    {
-        goto bad;
-    }
- 
-    if (sum == 0)
-    {
-        /*
-         *  If checksum is zero, by default libnet will compute a checksum
-         *  for the user.  The programmer can override this by calling
-         *  libnet_toggle_checksum(l, ptag, 1);
-         */
-        libnet_pblock_setflags(p, LIBNET_PBLOCK_DO_CHECKSUM);
-    }
+    LIBNET_BUILD_ICMP_ERR_FINISH(LIBNET_ICMPV4_TIMXCEED_H);
+
     return (ptag ? ptag : libnet_pblock_update(l, p, h,
             LIBNET_PBLOCK_ICMPV4_TIMXCEED_H));
 bad:
@@ -469,19 +386,14 @@ bad:
     return (-1);
 }
 
-
 libnet_ptag_t
 libnet_build_icmpv4_redirect(u_int8_t type, u_int8_t code, u_int16_t sum,
-            u_int32_t gateway, u_int16_t orig_len, u_int8_t orig_tos,
-            u_int16_t orig_id, u_int16_t orig_frag, u_int8_t orig_ttl,
-            u_int8_t orig_prot, u_int16_t orig_check, u_int32_t orig_src,
-            u_int32_t orig_dst, u_int8_t *payload, u_int32_t payload_s,
-            libnet_t *l, libnet_ptag_t ptag)
+u_int32_t gateway, u_int8_t *payload, u_int32_t payload_s, libnet_t *l,
+libnet_ptag_t ptag)
 
 {
     u_int32_t n, h;
-    libnet_ptag_t ipv4;
-    libnet_pblock_t *p, *q;
+    libnet_pblock_t *p;
     struct libnet_icmpv4_hdr icmp_hdr;
 
     if (l == NULL)
@@ -489,45 +401,13 @@ libnet_build_icmpv4_redirect(u_int8_t type, u_int8_t code, u_int16_t sum,
         return (-1);
     } 
 
-    p = NULL;
-    /* payload refers to the payload in the original IP header */
-    if ((payload && !payload_s) || (!payload && payload_s))
-    {
-         snprintf(l->err_buf, LIBNET_ERRBUF_SIZE,
-			     "%s(): payload inconsistency\n", __func__);
-        goto bad;
-    }
-
-    /*
-     *  Build the original IPv4 header in its own pblock.  We stick the
-     *  payload here.
-     */
-    ipv4 = libnet_build_ipv4(orig_len, orig_tos, orig_id, orig_frag, orig_ttl,
-        orig_prot, orig_check, orig_src, orig_dst, payload, payload_s, l, 0);
-
-    if (ipv4 == -1)
-    {
-       /* error set elsewhere */
-       return (-1);
-    }
-
-    if (orig_check == 0)
-    {
-       int c = 0;
-       l->aligner = 8 - (l->link_offset % 8);
-       q = libnet_pblock_find(l, ipv4);
-       c = libnet_do_checksum(l, q->buf, libnet_pblock_p2p(q->type),
-                              q->h_len);
-       if (c == -1)
-       {
-           /* err msg set in libnet_do_checksum() */
-           return (-1);
-       }
-    }
-
     n = LIBNET_ICMPV4_REDIRECT_H;               /* size of memory block */
-    /* hl for checksum */
-    h = LIBNET_ICMPV4_REDIRECT_H + LIBNET_IPV4_H + payload_s;
+    /* 
+     * FREDRAYNAL: as ICMP checksum includes what is embedded in 
+     * the payload, and what is after the ICMP header, we need to include
+     * those 2 sizes.
+     */
+    h = LIBNET_ICMPV4_REDIRECT_H + payload_s + l->total_size; 
 
     /*
      *  Find the existing protocol block if a ptag is specified, or create
@@ -542,25 +422,11 @@ libnet_build_icmpv4_redirect(u_int8_t type, u_int8_t code, u_int16_t sum,
     memset(&icmp_hdr, 0, sizeof(icmp_hdr));
     icmp_hdr.icmp_type      = type;             /* packet type */
     icmp_hdr.icmp_code      = code;             /* packet code */
-    icmp_hdr.icmp_sum       = sum;              /* checksum */
-    icmp_hdr.hun.gateway    = gateway;   /* gateway address */
+    icmp_hdr.icmp_sum       = (sum ? htons(sum) : 0);  /* checksum */
+    icmp_hdr.hun.gateway    = gateway;
+
+    LIBNET_BUILD_ICMP_ERR_FINISH(LIBNET_PBLOCK_ICMPV4_REDIRECT_H);
 
-    n = libnet_pblock_append(l, p, (u_int8_t *)&icmp_hdr, 
-                             LIBNET_ICMPV4_REDIRECT_H);
-    if (n == -1)
-    {
-        goto bad;
-    }
- 
-    if (sum == 0)
-    {
-        /*
-         *  If checksum is zero, by default libnet will compute a checksum
-         *  for the user.  The programmer can override this by calling
-         *  libnet_toggle_checksum(l, ptag, 1);
-         */
-        libnet_pblock_setflags(p, LIBNET_PBLOCK_DO_CHECKSUM);
-    }
     return (ptag ? ptag : libnet_pblock_update(l, p, h,
             LIBNET_PBLOCK_ICMPV4_REDIRECT_H));
 bad:
@@ -568,5 +434,4 @@ bad:
     return (-1);
 }
 
-
 /* EOF */
